///File : ConfigObj.cpp
//------------------------------------------------------------------------------
/*
    This file is part of MaxiShoot.

    MaxiShoot is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    MaxiShoot is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MaxiShoot.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "ConfigObj.hpp"

#include <sys/types.h>
#include <unistd.h>
//------------------------------------------------------------------------------

int ConfigObj::GetGameState() const
{
    return m_gameState;
}
//------------------------------------------------------------------------------
void ConfigObj::SetGameState(const int &state)
{
    m_gameState = state;
}
//------------------------------------------------------------------------------
bool ConfigObj::IsFpsOn() const
{
    return m_fpsOn;
}
//------------------------------------------------------------------------------
void ConfigObj::SetFpsOn(const bool &fps)
{
    m_fpsOn = fps;
}
//------------------------------------------------------------------------------
bool ConfigObj::IsFullScreenOn() const
{
    return m_fullScreenOn;
}
//------------------------------------------------------------------------------
void ConfigObj::SetFullScreenOn(const bool &fullscreen)
{
    m_fullScreenOn = fullscreen;
}
//------------------------------------------------------------------------------
std::string ConfigObj::GetCurrentVideoModeText() const
{
    std::string str = intToString(m_currentVideoMode.Width) + std::string("x") + intToString(m_currentVideoMode.Height);
    return str;
}
//------------------------------------------------------------------------------
void ConfigObj::ChangeVideoMode()
{
    for(int i = 0; i < sf::VideoMode::GetModesCount(); i++)
    {
        if(sf::VideoMode::GetMode(i) == m_currentVideoMode)
        {
            // Si c'est le dernier mode de la liste ==> Début.
            if(i == sf::VideoMode::GetModesCount() - 1)
            {
                m_currentVideoMode = sf::VideoMode::GetMode(0);
            }
            else
            {
                m_currentVideoMode = sf::VideoMode::GetMode(i+1);
            }

            return;
        }
    }
    return;
}
//------------------------------------------------------------------------------

sf::VideoMode ConfigObj::GetCurrentVideoMode() const
{
    return m_currentVideoMode;
}

//------------------------------------------------------------------------------
std::vector<std::string> ConfigObj::GetModeDirNamesList()
{
    return m_modeDirNamesList;
}
//------------------------------------------------------------------------------
void ConfigObj::SetCurrentModeDirName(const std::string &modeDirName)
{
    // 1  - Il existe ?
    int id = -1;

    for(int i = 0; i < m_modeDirNamesList.size(); i++)
    {
        if(m_modeDirNamesList[i] == modeDirName)
        {
            id = i;
            break;
        }
    }

    // On change
    if(id != -1)
    {
        m_modeDirName = modeDirName;
    }
}
//------------------------------------------------------------------------------
std::string ConfigObj::GetCurrentModeDirName()
{
    return m_modeDirName;
}
//------------------------------------------------------------------------------
std::string ConfigObj::GetTempModeTotalDir(const std::string &tmpModeName)
{
    return (m_dataDir + std::string("/modes/") + tmpModeName);
}
//------------------------------------------------------------------------------
std::string ConfigObj::GetNextMode(const std::string &modeDirName)
{
    if(m_modeDirNamesList.size() != 0)
    {
        // 1 On trouve l'id du mode
        int id = 0;

        for(int i = 0; i < m_modeDirNamesList.size(); i++)
        {
            if(m_modeDirNamesList[i] == modeDirName)
            {
                id = i;
                break;
            }
        }

        // Si id est le dernier de la liste, on revient au premier.
        if(id + 1 >= m_modeDirNamesList.size())
        {
            return m_modeDirNamesList[0];
        }
        else
        {
            // On retourne le suivant :
            return m_modeDirNamesList[id + 1];
        }
    }
    else
    {
        return "no-mode";
    }
}

//------------------------------------------------------------------------------
void ConfigObj::LoadModeDirNamesList()
{
    m_modeDirNamesList.clear();

    std::string filename = GetDataDir() + std::string("/mode-list.conf");

    std::ifstream file(filename.c_str(), std::ios::in);  // on ouvre en lecture

    if(file)  // si l'ouverture a fonctionné
    {
        std::string str;

        while(!file.eof())
        {
            std::getline(file, str);

            if(!str.empty())
            {
                m_modeDirNamesList.push_back(str);
            }
        }

        file.close();
    }
    else
    {
        ERROR_FLUX << "Can't load modes list !\nill now exit !" << std::endl;
        exit(-1);
    }
}
//------------------------------------------------------------------------------
std::string ConfigObj::GetDataDir()
{
    if(m_dataDir.empty())
    {
        FindStandardPaths();
    }

    return m_dataDir;
}
//------------------------------------------------------------------------------

std::string ConfigObj::GetUserConfigDir()
{
    if(m_userConfigDir.empty())
    {
        FindStandardPaths();
    }

    return m_userConfigDir;
}
//------------------------------------------------------------------------------
void ConfigObj::SaveScores()
{
        #if defined WIN32
        std::string filename = GetUserConfigDir() + std::string("/maxishoot.") + m_modeDirName + std::string(".score");
        #else
        std::string filename = std::string(LINUX_SCORE_FILE_DIR) + std::string("/maxishoot.") + m_modeDirName + std::string(".score");
        #endif

    std::ofstream file(filename.c_str(), std::ios::out | std::ios::trunc);  // on ouvre en ecriture

    if(file)
    {
        for(int i = 0; i < NB_SCORE_SAVED; i++)
        {
            file << m_scoresTbl[i].user << std::endl;
            file << intToString(m_scoresTbl[i].score) << std::endl;
        }

        file.close();
    }
    else
    {
        ERROR_FLUX << "Can't save scores \"" << filename << "\"" << std::endl;
    }
}
//------------------------------------------------------------------------------
std::string ConfigObj::GetScores()
{
    std::string scoresStr;

    for(int i = 0; i < NB_SCORE_SAVED; i++)
    {
        scoresStr += intToString(i + 1) + std::string(") ") + m_scoresTbl[i].user + std::string(" : ") + intToString(m_scoresTbl[i].score) + std::string("\n");
    }

    return scoresStr;
}
//------------------------------------------------------------------------------

void ConfigObj::LoadScores()
{
    for(int i = 0; i < NB_SCORE_SAVED; i++)
    {
        m_scoresTbl[i].score = 0;
        m_scoresTbl[i].user = "Pas de score";
    }

        #if defined WIN32
        std::string filename = GetUserConfigDir() + std::string("/maxishoot.") + m_modeDirName + std::string(".score");
        #else
        std::string filename = std::string(LINUX_SCORE_FILE_DIR) + std::string("/maxishoot.") + m_modeDirName + std::string(".score");
        #endif

    std::ifstream file(filename.c_str(), std::ios::in);  // on ouvre en lecture

    if(file)  // si l'ouverture a fonctionné
    {
        std::string str;

        for(int i = 0; i < NB_SCORE_SAVED && !file.eof(); i++)
        {
            std::getline(file, str);

            if(!str.empty())
            {
                m_scoresTbl[i].user = str;
            }
            else
            {
                m_scoresTbl[i].user = "Pas de score";
            }

            if(!file.eof())
            {
                std::getline(file, str);
                if(!str.empty())
                {
                    m_scoresTbl[i].score = stringToInt(str);
                }
                else
                {
                    m_scoresTbl[i].score = 0;
                }
            }
        }

        file.close();
    }
    else
    {
        ERROR_FLUX << "Can't load scores \"" << filename << "\"" << std::endl;
    }

    std::sort(m_scoresTbl, m_scoresTbl + NB_SCORE_SAVED, compareScore);
}
//------------------------------------------------------------------------------
bool compareScore(const StructScore &i, const StructScore &j)
{
    return i.score > j.score;
}
//------------------------------------------------------------------------------
bool ConfigObj::AddScore(const int &score)
{
    if(m_scoresTbl[NB_SCORE_SAVED - 1].score < score)
    {
        m_scoresTbl[NB_SCORE_SAVED - 1].score = score;

        #if defined (WIN32)
        char user[100];
        DWORD size = 100;
        GetUserName(user,&size);

        m_scoresTbl[NB_SCORE_SAVED - 1].user = std::string(user);
        #else
        m_scoresTbl[NB_SCORE_SAVED - 1].user = getenv("USER");
        #endif


        std::sort(m_scoresTbl, m_scoresTbl + NB_SCORE_SAVED, compareScore);
        return true;
    }
    return false;
}
//------------------------------------------------------------------------------
void ConfigObj::FindStandardPaths()
{
    std::string exeName, exeDir, exeTotalPath;

    #if defined (WIN32)
    // Data dir
    char exepath[4096] = {0};
    GetModuleFileName(NULL, exepath, sizeof(exepath));  /* get fullpath of the service */

    exeTotalPath = exepath;
    exeName = exeTotalPath.substr(exeTotalPath.rfind('\\') + 1);
    exeDir = exeTotalPath.substr(0,exeTotalPath.rfind('\\') + 1);

    m_dataDir = exeDir + std::string("data");

    // User config dir
    m_userConfigDir = m_dataDir;
    #else
    // Data dir
    char exepath[PATH_MAX] = {0};
    char procid[PATH_MAX] = {0};
    char exelink[PATH_MAX] = {0};

    sprintf(procid, "%u", getpid());

    strcpy(exelink, "/proc/");
    strcat(exelink, procid);
    strcat(exelink, "/exe");

    int result = readlink(exelink, exepath, sizeof(exepath));
    if ( result != -1 )
    {
        exepath[result] = '\0'; // readlink() doesn't NUL-terminate the buffer
    }

    exeTotalPath = exepath;

    exeName = exeTotalPath.substr(exeTotalPath.rfind('/') + 1);
    exeDir = exeTotalPath.substr(0,exeTotalPath.rfind('/') + 1);

    if(exeDir.substr(exeDir.size() - 4) == std::string("bin/") || exeDir.substr(exeDir.size() - 6) == std::string("games/"))
    {
        m_dataDir = exeDir + std::string("../share/") + exeName + std::string("/data");
    }
    else
    {
        m_dataDir = exeDir + std::string("data");
    }

    // User config dir
    std::string userDir = getenv("HOME");
    m_userConfigDir = userDir + std::string("/.") + exeName;

    std::string command = std::string("/usr/bin/test ! -d ") + m_userConfigDir +  std::string(" && /bin/mkdir ") + m_userConfigDir;
    result = system(command.c_str());
    #endif
}
//------------------------------------------------------------------------------
void ConfigObj::SaveAll()
{
    // Scores
    SaveScores();

    // Config
    std::string filename = GetUserConfigDir() + std::string("/conf");

    std::ofstream file(filename.c_str(), std::ios::out | std::ios::trunc);  // on ouvre en ecriture

    if(file)
    {
        file << m_RcManager.IsMusicOn() << std::endl;
        file << m_RcManager.IsSoundOn() << std::endl;
        file << IsFpsOn() << std::endl;
        file << m_modeDirName << std::endl;
        file << IsFullScreenOn() << std::endl;
        file << m_currentVideoMode.Width << std::endl;
        file << m_currentVideoMode.Height << std::endl;

        file.close();
    }
    else
    {
        ERROR_FLUX << "Can't save config \"" << filename << "\"" << std::endl;
    }
}
//------------------------------------------------------------------------------
void ConfigObj::LoadConfig()
{
    INFO_FLUX << std::endl;
    INFO_FLUX << "--------------------------------------------" << std::endl;
    INFO_FLUX << "-                New Mode                  -" << std::endl;
    INFO_FLUX << "--------------------------------------------" << std::endl;

    // Base
    FindStandardPaths();

    // Conf
    std::string filename = GetUserConfigDir() + std::string("/conf");

    std::ifstream file(filename.c_str(), std::ios::in);  // on ouvre en lecture

    if(file)  // si l'ouverture a fonctionné
    {
        std::string str;

        std::getline(file, str);
        m_RcManager.SetMusicOn(str == "1");

        std::getline(file, str);
        m_RcManager.SetSoundOn(str == "1");

        std::getline(file, str);
        SetFpsOn(str == "1");

        std::getline(file, str);
        SetCurrentModeDirName(str);

        std::getline(file, str);
        SetFullScreenOn(str == "1");

        sf::VideoMode tempVideoMode(sf::VideoMode::GetDesktopMode());
        std::getline(file, str);
        tempVideoMode.Width = stringToInt(str);
        std::getline(file, str);
        tempVideoMode.Height = stringToInt(str);

        if(tempVideoMode.IsValid())
        {
            m_currentVideoMode = tempVideoMode;
        }
        else
        {
            m_currentVideoMode = sf::VideoMode::GetDesktopMode();
        }

        file.close();
    }
    else
    {
        ERROR_FLUX << "Can't open config file \"" << filename << "\"\n\\==> Load default conf." << std::endl;

        m_RcManager.SetMusicOn(true);
        m_RcManager.SetSoundOn(true);
        SetFpsOn(false);
        SetFullScreenOn(true);
        m_currentVideoMode = sf::VideoMode::GetDesktopMode();
    }
}
//------------------------------------------------------------------------------
void ConfigObj::LoadMode()
{
    // Mode (cas ou)
    if(m_modeDirName.empty())
    {
        m_modeDirName = "Normal";
    }

    m_gameState = GAME_REQUEST_NEW;

    // Files
    std::string modeDir = m_dataDir + std::string("/modes/") + m_modeDirName;
    m_RcManager.LoadNewMode(modeDir);

    // Scores
    LoadScores();
}
//------------------------------------------------------------------------------

RcManager *ConfigObj::GetRcManager()
{
    return &m_RcManager;
}
//------------------------------------------------------------------------------

