///File : GameScreen.hpp
//------------------------------------------------------------------------------
/*
    This file is part of MaxiShoot.

    MaxiShoot is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    MaxiShoot is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with MaxiShoot.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#if not defined GAME_SCREEN_HPP_MAXI_SHOOT
#define GAME_SCREEN_HPP_MAXI_SHOOT
//------------------------------------------------------------------------------
#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>

#include <complex>
//------------------------------------------------------------------------------
#include "myConf.hpp"
#include "Screen.hpp"
#include "main.hpp"
//------------------------------------------------------------------------------

#define RAD_TO_DEG 57.295779579
#define DEG_TO_RAD 0.017453292

#define ITEMS_ERASE_BORDER 100.f

#define GUN_ROTATION_MOUSE_SPEED 0.099
#define GUN_ROTATION_KEYBOARD_SPEED 90.f

struct StructGun
{
    sf::Sprite GunSpr;
    sf::Sprite BunkerSpr;

    bool m_isHuman;

    float m_timeBeforeLastShot;
    int m_simultaneousBullet;

    float m_timeBeforeLastLaser;
    float m_timeBeforeLastLaserBullet;
    float m_laserBulletLeft;

    bool isSight;
};

struct StructDebris
{
    sf::Sprite spr;
    sf::Vector2<float> speed;
    float lifeTimeLeft, lifeTime;
};

struct StructBullet
{
    sf::Sprite spr;
    bool isLaser;
};

struct StructBurstInfo
{
    sf::Vector2<float> pos;
    float angleMin;
    float angleMax;
    bool isOnShield;
    bool isLife;
    bool isMini;
};

class GameScreen : public Screen
{
public:
    GameScreen(ConfigObj *configObj);
    virtual int Run (sf::RenderWindow &App);

protected:
    void RotateGun(const int &gunId, const float &rotation);
    void Shoot(const int &gunId, const bool &isLaser = false);
    void NewGame();
    void BulletMove(const float &frameTime);
    void EnemiesMove(const float &frameTime);
    void DebrisMove(const float &frameTime);
    void CreateExplosion(const StructBurstInfo &burstInfo);
    void AddEnemies();
    void NewGun(const float &gunXPos, const bool &isHuman = true);
    void IAComputerPlayer(const float &frameTime);

    std::vector<StructBullet> m_BulletsList;
    std::vector<StructEnemy> m_Enemies;
    std::vector<StructDebris> m_Debris;
    std::vector<StructGun> m_Guns;

    bool m_scoreSent;
    bool m_haveScore;

    float m_shieldAlpha;
};
#endif // GAME_SCREEN_HPP_MAXI_SHOOT
